VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "ApiPrintJob"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

'**************************************
'Windows API/Global Declarations for :Ge
'     tJob - Getting job info from the printer
'     queue
'**************************************
'\\ --[ApiPrintJob]---------------------
'     ------------------------------
'\\ Object wrapper for the API structure
'     s and calls exported by
'\\ "winspool.drv" pertaining to Printer
'     Jobs
'\\ ------------------------------------
'     ------------------------------
'\\ (c) 2001 - Merrion Computing.
'\\ All rights to use, reproduce or publ
'     ish this code reserved
'\\ Please check http://www.merrioncompu
'     ting.com for updates.
'\\ ------------------------------------
'     -------------------------------
'\\ Sourced from: winspool.h


Public Enum PrintJobStatuses
    JOB_STATUS_PAUSED = &H1
    JOB_STATUS_ERROR = &H2
    JOB_STATUS_DELETING = &H4
    JOB_STATUS_SPOOLING = &H8
    JOB_STATUS_PRINTING = &H10
    JOB_STATUS_OFFLINE = &H20
    JOB_STATUS_PAPEROUT = &H40
    JOB_STATUS_PRINTER = &H80
    JOB_STATUS_DELETED = &H100
    JOB_STATUS_BLOCKED_DEVICEQUEUE = &H200
    JOB_STATUS_USER_INTERVENTION = &H400
    JOB_STATUS_RESTART = &H800
End Enum


Private Type SYSTEMTIME
    wYear As Integer
    wMonth As Integer
    wDayOfWeek As Integer
    wDay As Integer
    wHour As Integer
    wMinute As Integer
    wSecond As Integer
    wMilliseconds As Integer
End Type


Private Type JOB_INFO_1
    JobId As Long
    lpPrinterName As String
    lpMachinename As String
    lpUserName As String
    lpDocumentName As String
    lpDataType As String
    lpStatus As String
    Status As PrintJobStatuses
    Priority As Long
    Position As Long
    TotalPages As Long
    PagesPrinted As Long
    Submitted As SYSTEMTIME
End Type


Private Type JOB_INFO_2
    JobId As Long
    lpPrinterName As String
    lpMachinename As String
    lpUserName As String
    lpDocumentName As String
    lpNotifyName As String
    lpDataType As String
    lpPrintProcessor As String
    lpParameters As String
    lpDriverName As String
    lpDevmode As Long 'Pointer to DevMode
    lpStatus As String
    lpSecurityDescriptor As Long 'Pointer to SECURITY_DESCRIPTOR
    Status As PrintJobStatuses
    Priority As Long
    Position As Long
    StartTime As Long
    UntilTime As Long
    TotalPages As Long
    JobSize As Long
    Submitted As SYSTEMTIME
    time As Long
    PagesPrinted As Long
End Type
    '\\ Included in WINNT4 and beyond


Private Type JOB_INFO_3
    JobId As Long
NextJobId As Long
Reserved As Long 'must be set to zero
End Type


Private Declare Function GetJob Lib "winspool.drv" Alias "GetJobA" _
    (ByVal hPrinter As Long, _
    ByVal JobId As Long, _
    ByVal Level As Long, _
    buffer As Long, _
    ByVal pbSize As Long, _
    pbSizeNeeded As Long) As Long
    '\\ Private member variables
    Private mJobId As Long 'Unique id of this job
    Private mhPrinter As Long 'Printer this job is on...
    Private mJOB_INFO_1 As JOB_INFO_1
    Private mJOB_INFO_2 As JOB_INFO_2
    Private mJOB_INFO_3 As JOB_INFO_3
    '\\ Private variable to prevent unnecces
    '     sary calls to the dll...
    Private mLastApiCall(1 To 3) As Date



Public Property Get DataType() As String


    If mhPrinter <> 0 And mJobId <> 0 Then
        Call RefreshJobInfo(1)
        DataType = mJOB_INFO_1.lpDataType
    End If
End Property


Public Property Get DocumentName() As String


    If mhPrinter <> 0 And mJobId <> 0 Then
        Call RefreshJobInfo(1)


        DocumentName = mJOB_INFO_1.lpDocumentName
        End If
    End Property
Friend Property Let JobId(ByVal newJobId As Long)
mJobId = newJobId
End Property


Public Property Get JobId() As Long
    JobId = mJobId
End Property


Public Property Get MachineName() As String


    If mhPrinter <> 0 And mJobId <> 0 Then
        Call RefreshJobInfo(1)
        MachineName = mJOB_INFO_1.lpMachinename
    End If
End Property


Public Property Get PagesPrinted() As Long


    If mhPrinter <> 0 And mJobId <> 0 Then
        Call RefreshJobInfo(1)
        PagesPrinted = mJOB_INFO_1.PagesPrinted
    End If
End Property


Public Property Get Position() As Long


    If mhPrinter <> 0 And mJobId <> 0 Then
        Call RefreshJobInfo(1)
        Position = mJOB_INFO_1.Position
    End If
End Property
Friend Property Let PrinterHandle(ByVal newhPrinter As Long)
mhPrinter = newhPrinter
End Property


Public Property Get PrinterName() As String


    If mhPrinter <> 0 And mJobId <> 0 Then
        Call RefreshJobInfo(1)
        PrinterName = mJOB_INFO_1.lpPrinterName
    End If
End Property


Public Property Get Priority() As Long


    If mhPrinter <> 0 And mJobId <> 0 Then
        Call RefreshJobInfo(1)
        Priority = mJOB_INFO_1.Priority
    End If
End Property


Private Sub RefreshJobInfo(ByVal Index As Integer)
    Dim lRet As Long
    Dim SizeNeeded As Long
    Dim buffer() As Long
    '\\ Prevent unneccesary call to the API


    If DateDiff("s", mLastApiCall(Index), Now) > 1 Then
        mLastApiCall(Index) = Now
        ReDim Preserve buffer(0 To 1) As Long
        lRet = GetJob(mhPrinter, mJobId, Index, buffer(0), UBound(buffer), SizeNeeded)


        If SizeNeeded > 0 Then
            ReDim Preserve buffer(0 To (SizeNeeded / 4) + 3) As Long
            lRet = GetJob(mhPrinter, mJobId, Index, buffer(0), UBound(buffer) * 4, SizeNeeded)


            If Index < 1 Or Index > 3 Then
                Debug.Print "Error in call to ApiPrintJob:RefreshJobInfo - invalid index"
            Else
                Select Case Index
                    Case 1
                        With mJOB_INFO_1
                            .JobId = buffer(0)
                            .lpPrinterName = StringFromPointer(buffer(1), 1024)
                            .lpMachinename = StringFromPointer(buffer(2), 1024)
                            .lpUserName = StringFromPointer(buffer(3), 1024)
                            .lpDocumentName = StringFromPointer(buffer(4), 1024)
                            .lpDataType = StringFromPointer(buffer(5), 1024)
                            .lpStatus = StringFromPointer(buffer(6), 1024)
                            .Status = buffer(7)
                            .Priority = buffer(8)
                            .Position = buffer(9)
                            .TotalPages = buffer(10)
                            .PagesPrinted = buffer(11)
                            'Submitted is also here...
                            With .Submitted
                                .wYear = LoWord(buffer(12))
                                .wMonth = HiWord(buffer(12))
                                .wDayOfWeek = LoWord(buffer(13))
                                .wDay = HiWord(buffer(13))
                                .wHour = LoWord(buffer(14))
                                .wMinute = HiWord(buffer(14))
                                .wSecond = LoWord(buffer(15))
                                .wMilliseconds = HiWord(buffer(15))
                            End With
                        End With
                    Case 2
                        With mJOB_INFO_2
                            .JobId = buffer(0)
                            .lpPrinterName = StringFromPointer(buffer(1), 1024)
                            .lpMachinename = StringFromPointer(buffer(2), 1024)
                            .lpUserName = StringFromPointer(buffer(3), 1024)
                            .lpDocumentName = StringFromPointer(buffer(4), 1024)
                            .lpNotifyName = StringFromPointer(buffer(5), 1024)
                            .lpDataType = StringFromPointer(buffer(6), 1024)
                            .lpPrintProcessor = StringFromPointer(buffer(7), 1024)
                            .lpParameters = StringFromPointer(buffer(8), 1024)
                            .lpDriverName = StringFromPointer(buffer(9), 1024)
                            .lpDevmode = buffer(10) 'To do: Replace with ApiDevMode class when available
                            .lpStatus = StringFromPointer(buffer(11), 1024)
                            .lpSecurityDescriptor = buffer(12) 'To do: Replace...
                            .Status = buffer(13)
                            .Priority = buffer(14)
                            .Position = buffer(15)
                            .StartTime = buffer(16)
                            .UntilTime = buffer(17)
                            .TotalPages = buffer(18)
                            .JobSize = buffer(19)
                            'Submitted is here....
                            With .Submitted
                                .wYear = LoWord(buffer(20))
                                .wMonth = HiWord(buffer(20))
                                .wDayOfWeek = LoWord(buffer(21))
                                .wDay = HiWord(buffer(21))
                                .wHour = LoWord(buffer(22))
                                .wMinute = HiWord(buffer(22))
                                .wSecond = LoWord(buffer(23))
                                .wMilliseconds = HiWord(buffer(23))
                            End With
                            .time = buffer(24)
                            .PagesPrinted = buffer(25)
                        End With
                        
                    Case 3
                        With mJOB_INFO_3
                            .JobId = buffer(0)
                            .NextJobId = buffer(1)
                            .Reserved = buffer(2)
                        End With
                End Select
        End If
    End If
End If
End Sub


Public Property Get Status() As PrintJobStatuses


    If mhPrinter <> 0 And mJobId <> 0 Then
        Call RefreshJobInfo(1)
        Status = mJOB_INFO_1.Status
    End If
End Property


Public Property Get Submitted() As Date
Dim dtSubmitted As Date
    
    Call RefreshJobInfo(1)
    With mJOB_INFO_1.Submitted
        dtSubmitted = DateSerial(.wYear, .wMonth, .wDay)
        dtSubmitted = dtSubmitted + TimeSerial(.wHour, .wMinute, .wSecond)
    End With

    Submitted = dtSubmitted
    
End Property


Public Property Get TotalPages() As Long


    If mhPrinter <> 0 And mJobId <> 0 Then
        Call RefreshJobInfo(1)
        TotalPages = mJOB_INFO_1.TotalPages
    End If
End Property


Public Property Get StartTime() As Long
    Call RefreshJobInfo(2)
    StartTime = mJOB_INFO_2.StartTime
End Property


Public Property Get UntilTime() As Long
    Call RefreshJobInfo(2)
    UntilTime = mJOB_INFO_2.UntilTime
End Property


Public Property Get Username() As String
    Call RefreshJobInfo(1)
    Username = mJOB_INFO_1.lpUserName
End Property



