VERSION 5.00
Begin VB.UserControl PrinterWatch 
   ClientHeight    =   3945
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   7785
   ScaleHeight     =   3945
   ScaleWidth      =   7785
   ToolboxBitmap   =   "PrinterWatch.ctx":0000
   Begin VB.ListBox lbPrinterData 
      Columns         =   5
      Height          =   2985
      Left            =   0
      TabIndex        =   0
      Top             =   0
      Width           =   7575
   End
End
Attribute VB_Name = "PrinterWatch"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

Private WithEvents mPrinter As ApiPrinter
Attribute mPrinter.VB_VarHelpID = -1

Private mPrinterName As String

Public Event JobAdded()
Public Event JobSet()
Public Event JobDeleted()
Public Event JobWritten()
Public Event JobChanged()

Private Declare Function ShellExecute Lib "shell32.dll" Alias "ShellExecuteA" (ByVal hwnd As Long, ByVal lpOperation As String, ByVal lpFile As String, ByVal lpParameters As String, ByVal lpDirectory As String, ByVal nShowCmd As Long) As Long

Private Const SW_SHOW = 5

Public Property Get AveragePagesPerMonth() As Long

    If Not mPrinter Is Nothing Then
        AveragePagesPerMonth = mPrinter.AveragePagesPerMonth
    End If
    
End Property


Public Property Get Comment() As String

    If Not mPrinter Is Nothing Then
        Comment = mPrinter.Comment
    End If

End Property

Public Property Get DataType() As String

    If Not mPrinter Is Nothing Then
        DataType = mPrinter.DataType
    End If

End Property


Public Property Get DefaultPriority() As Long

    If Not mPrinter Is Nothing Then
        DefaultPriority = mPrinter.DefaultPriority
    End If

End Property

Public Property Get Description() As String

    If Not mPrinter Is Nothing Then
        Description = mPrinter.Description
    End If
    
End Property



Public Property Get hwndCommunicator() As Long

Static mHwnd As Long

If mHwnd = 0 Then
    mHwnd = PrinterIPC.CreateCommunicationWindow(True)
End If
hwndCommunicator = mHwnd

End Property

Public Property Let PrinterName(ByVal newName As String)

If newName <> mPrinterName Then
    mPrinterName = newName
    mPrinter.DeviceName = newName
End If

End Property


Public Property Get PrintProcessor() As String

    If Not mPrinter Is Nothing Then
        PrintProcessor = mPrinter.PrintProcessor
    End If

End Property

Public Property Get Priority() As Long

    If Not mPrinter Is Nothing Then
        Priority = mPrinter.Priority
    End If

End Property

Public Property Get Queued() As Boolean

    If Not mPrinter Is Nothing Then
        Queued = mPrinter.Queued
    End If

End Property

Private Sub RefreshJobList()

Dim jobThis As ApiPrintJob

With lbPrinterData
    .Clear
    For Each jobThis In mPrinter.PrintJobs
        .AddItem jobThis.DocumentName & vbKeyTab & jobThis.Username & vbKeyTab & jobThis.MachineName & vbKeyTab & jobThis.PagesPrinted
    Next jobThis
End With

End Sub

Public Property Get SeparatorFile() As String

    If Not mPrinter Is Nothing Then
        SeparatorFile = mPrinter.SeparatorFile
    End If

End Property

Public Property Get ServerName() As String

    If Not mPrinter Is Nothing Then
        ServerName = mPrinter.ServerName
    End If

End Property


Public Property Get ShareName() As String

    If Not mPrinter Is Nothing Then
        ShareName = mPrinter.ShareName
    End If
    
End Property


'\ --[Status]---------------------------------------------------------------------------------
'\ Returns the status of the selected printer when it is called.
'\ -------------------------------------------------------------------------------------------
Public Property Get Status() As Printer_Status

    If Not mPrinter Is Nothing Then
        Status = mPrinter.Status
    End If
    
End Property
Public Property Get Direct() As Boolean

    If Not mPrinter Is Nothing Then
        Direct = mPrinter.Direct
    End If

End Property

Public Property Get DriverName() As String

    If Not mPrinter Is Nothing Then
        DriverName = mPrinter.DriverName
    End If

End Property

Public Property Get Location() As String

    If Not mPrinter Is Nothing Then
        Location = mPrinter.Location
    End If

End Property

Public Property Get Parameters() As String

    If Not mPrinter Is Nothing Then
        Parameters = mPrinter.Parameters
    End If

End Property

Public Property Get PendingJobsCount() As Long

    If Not mPrinter Is Nothing Then
        PendingJobsCount = mPrinter.PendingJobsCount
    End If

End Property

Public Property Get PortName() As String

    If Not mPrinter Is Nothing Then
        PortName = mPrinter.PortName
    End If

End Property
Public Property Get PrinterName() As String

    PrinterName = mPrinterName
    
End Property


Public Sub TriggerEvent(ByVal dwState As Long)

Dim bRefresh As Long

'\\ dwState can be any combination of PrinterChangeNotifications
'\\ therefore we need to decode the ones we are interested in..
If dwState And PRINTER_CHANGE_ADD_JOB Then
    RaiseEvent JobAdded
End If

If dwState And PRINTER_CHANGE_SET_JOB Then
    RaiseEvent JobSet
End If

If dwState And PRINTER_CHANGE_DELETE_JOB Then
    RaiseEvent JobDeleted
End If

If dwState And PRINTER_CHANGE_WRITE_JOB Then
    RaiseEvent JobWritten
End If

If dwState And PRINTER_CHANGE_JOB Then
    bRefresh = True
    RaiseEvent JobChanged
End If
     
Call RefreshJobList
    
End Sub

Private Sub UserControl_Initialize()

'\\ If we can't find an existing "PrintWatchServer" then start one
If FindWindow(vbNullString, WINDOWTITLE_SERVER) = 0 Then
    Call ShellExecute(0, "Open", App.Path & "\PrintWatchServer.exe", "", "", SW_SHOW)
    While FindWindow(vbNullString, WINDOWTITLE_SERVER) = 0
        DoEvents
    Wend
End If

Set mPrinter = New ApiPrinter
Call InitialiseInterProcessCommunication(True)

'\\ Add me to the list of all controls...
If AllControls Is Nothing Then
    Set AllControls = New Collection
End If
AllControls.Add Me, "hwnd:" & Me.hwndCommunicator
mPrinter.CommunicatorWindow = Me.hwndCommunicator

Call RefreshJobList

End Sub

Private Sub UserControl_Resize()

With lbPrinterData
    .Width = UserControl.Width
    .Height = UserControl.Height
End With

End Sub


Private Sub UserControl_Terminate()

Set mPrinter = Nothing
Call TerminateInterProcessCommunication(Me.hwndCommunicator)
Call AllControls.Remove("hwnd:" & Me.hwndCommunicator)

End Sub
